

// Copyright (c) 2003 Michael Feathers
// Based on FIT by Cunningham & Cunningham, Inc
// Released under the terms of the GNU General Public License version 2 or later.	

#ifndef FIXTURE_H
#define FIXTURE_H

#include "MemberAdapter.h"
#include "Parse.h" // for ParsePtr MCF

#define PUBLISH(testClass,type,name)\
		adapters.push_back(new MemberAdapter<testClass,type>(#name, &testClass::name, this));

#include <vector>
#include <string>
#include <map>

class TypeAdapter;
class FixtureMaker;

using namespace std;


class Fixture
{
public:
								Fixture();
	virtual						~Fixture();

	virtual void				doTables(ParsePtr tables);

	virtual TypeAdapter			*adapterFor(const string& member) const;
	virtual bool				hasAdapterFor(const string& member) const;

	virtual Fixture				*makeFixture(const string& name) const;

	virtual void				doTable(ParsePtr table);
	virtual void				doRows(ParsePtr rows);
	virtual void				doRow(ParsePtr row);
	virtual void				doCells(ParsePtr cells);
	virtual void				doCell(ParsePtr cell, int columnNumber);

	virtual void				handleException(ParsePtr cell, exception& e);
	virtual void				check(ParsePtr cell, TypeAdapter *a);

	virtual void				setMaker(FixtureMaker *maker);

	static void					ignore(ParsePtr cell);
    static void					right(ParsePtr cell);
    static void					wrong(ParsePtr cell);
    static void					wrong(ParsePtr cell, string actual);
	static void					color(ParsePtr cell, string colorCode);

    static string				label(string s);
    static string				escape(string s);
    static string				escape(string s, char from, string to);
	static bool					endsWith(string s, string suffix);
	static string				counts();

	static string				currentTime();



	static const string			rightColor;
	static const string			wrongColor;
	static const string			exceptionColor;
	static const string			ignoreColor;

	static int					rights;
	static int					wrongs;
	static int					ignores;
	static int					exceptions;

	static map<string,string>	summary;

	vector<TypeAdapter *>		adapters;
	FixtureMaker				*maker;

private:
								Fixture(const Fixture&);
	Fixture&					operator=(Fixture&);


};


#endif