
// Copyright (c) 2003 Michael Feathers
// Based on FIT by Cunningham & Cunningham, Inc
// Released under the terms of the GNU General Public License version 2 or later.	

#ifndef MEMBERADAPTER_H
#define MEMBERADAPTER_H

#include "TypeAdapter.h"

#include <strstream>
#include <string>

using namespace std;


template<typename AFixture, typename AType> 
class MemberAdapter : public TypeAdapter
{
	typedef AType (AFixture::*MemberFunction)();
	typedef AType (AFixture::*MemberVariable);


public:
	MemberAdapter(const string& name, MemberFunction function, AFixture *fixture) 
		: TypeAdapter(name), function(function), variable(0), fixture(fixture)
	{
	}

	MemberAdapter(const string& name, MemberVariable variable, AFixture *fixture) 
		: TypeAdapter(name), function(0), variable(variable), fixture(fixture)
	{
	}


	virtual void invoke(const vector<string>& args = TypeAdapter::noArgs) 
	{
		value = (fixture->*function)();
	}

	virtual void set(const string& newValue)
	{
		strstream stage;

		stage << newValue;
		stage >> value;
		fixture->*variable = value;
	}

	virtual string valueAsString()
	{ 
		if (isObject()) {
			value = fixture->*variable;
		}
		else {
			invoke(); 
		}

		strstream stage;
		string s;

		stage << value;
		stage >> s;
		return s;
	}

	virtual bool isObject() const 
	{ 
		return variable != 0; 
	}


private:
	MemberFunction		function;
	MemberVariable		variable;
	AType				value; 
	AFixture			*fixture;


};


#endif
