
// Copyright (c) 2003 Michael Feathers
// Based on FIT by Cunningham & Cunningham, Inc
// Released under the terms of the GNU General Public License version 2 or later.	

#include <Platform.h>

#include <TestHarness.h>

#include <ActionFixture.h>
#include <Parse.h>
#include <TestFixtureMaker.h>
#include <IllegalAccessException.h>
#include <TypeAdapter.h>


template<typename AFixture>
class CallAdapter : public TypeAdapter
{
	typedef void (AFixture::*Call)(int);

public:
	CallAdapter(const string& name, Call call, AFixture *fixture) 
		: TypeAdapter(name), call(call), fixture(fixture)
	{
	}


	virtual void set(const string& newValue)
	{
	}

	virtual void invoke(const vector<string>& args = noArgs)
	{
		if(args.size() != 1)
			throw IllegalAccessException("Expected one argument in CallAdapter");
		strstream stream;
		int value;
		stream << args[0];
		stream >> value;
		(fixture->*call)(value);
	}

	virtual string valueAsString()
	{
		return "";
	}

	virtual bool isObject() const
	{
		return false;
	}

private:
	Call				call;
	AFixture			*fixture;

};


#define PUBLISH_CALL(testClass,type,name)\
		adapters.push_back(new CallAdapter<testClass>(#name, &testClass::name, this));


class AccountFixture : public Fixture
{
public:

	AccountFixture() 
		: balance (0) 
	{
		PUBLISH_CALL(AccountFixture,void,deposit);
		PUBLISH(AccountFixture,int,balance);
	}

	void deposit(int amount) 
	{
		balance += amount;
	}

	int balance;

};


class ActionTestMaker : public TestFixtureMaker
{
	virtual Fixture *make(const string& name) const
	{
		if (name == "AccountFixture")
			return new AccountFixture;
		return TestFixtureMaker::make(name);
	}

};


TEST(create,ActionFixture)
{

	Parse p(
		"<table><TR><TD>ActionFixture</TD></TR>"
		"<TR><TD>start</TD><TD>AccountFixture</TD><TD></TD></TR>"
		"<TR><TD>enter</TD><TD>deposit</TD><TD>5</TD></TR>"
		"<TR><TD>check</TD><TD>balance</TD><TD>5</TD></TR>"
		"</table>");

	Fixture fixture;
	ActionTestMaker maker;

	fixture.setMaker(&maker);
	fixture.doTables(&p);

	string out;

	p.print(out);
	CHECK(string::npos != out.find(Fixture::rightColor))

	delete ActionFixture::actor;

}


