
// Copyright (c) 2003 Michael Feathers
// Based on FIT by Cunningham & Cunningham, Inc
// Released under the terms of the GNU General Public License version 2 or later.	

#include <Platform.h>

#include <vector>
#include <cstdio>

#include <TestHarness.h>
#include <EnableMsMemoryLeakWarning.h>
#include <Parse.h>
#include <Fixture.h>
#include <TestFixtureMaker.h>
#include <TestFixtureMaker.h>

#include "TestRowFixture.h"


class ExpandedFixtureMaker : public TestFixtureMaker
{
	virtual Fixture *make(const string& name) const
	{
		if (name == "TestRowFixture")
			return new TestRowFixture;
		return TestFixtureMaker::make(name);
	}

};


string stringOf(string fileName) 
{
	string content;

	FILE *fp = fopen(fileName.c_str(), "r");

	if (fp) {
		int c;
		while (EOF != (c = fgetc(fp)))
			content += c;
		fclose(fp);
	}
	return content;
}


int main(int ac, char **av) 
{
	if (ac != 2) {
		EnableMsMemoryLeakWarning();

		fprintf(stderr,"Usage: fit <input-HTML-file>\n\n");
		fprintf(stderr,"Not enough arguments, running fit unit tests...\n");

		TestResult result;
		TestRegistry::runAllTests(result);
		exit(0);
	}

	try {
		Parse p(stringOf(av[1]));

		Fixture fixture;
		ExpandedFixtureMaker maker;

		fixture.setMaker(&maker);
		fixture.doTables(&p);

		string out;
		p.print(out);

		fprintf(stdout,"%s",out.c_str());
	} catch(exception& e) {
		fprintf(stderr,"Unexpected exception: %s\n", e.what());
	}

	return 0;
}


