
// Copyright (c) 2003 Michael Feathers
// Based on FIT by Cunningham & Cunningham, Inc
// Released under the terms of the GNU General Public License version 2 or later.	

#include <Platform.h>
#include "DynamicMaker.h"

#include <windows.h>

#include <ResolutionException.h>
#include <ActionFixture.h>
#include <ColumnFixture.h>
#include <PrimitiveFixture.h>
#include <Summary.h>
#include "../eg/ArithmeticFixture.h"


Fixture *DynamicMaker::make(const string& name) const
{
	Fixture *fixture = makeStaticFixture(name);

	if (fixture)
		return fixture;

	typedef Fixture *(*MAKER)(const char *); 

	string		libraryName		= splitName(name).first;
	string		fixtureName		= splitName(name).second;
    HINSTANCE	library			= LoadLibrary(libraryName.c_str()); 
  
    if (library == 0)
		throw ResolutionException("Unable to load \"" 
			+ libraryName 
			+ "\"");

    MAKER maker = (MAKER)GetProcAddress(library, "fixtureMaker");
	
	if (maker == 0)
		throw ResolutionException("Unable to find entry point in \"" 
			+ libraryName 
			+ "\"");
 
	fixture = maker(fixtureName.c_str()); 

	if (fixture == 0)
		throw ResolutionException("Unable to find fixture \"" 
			+ fixtureName
			+ "\" in \""
			+ libraryName
			+ "\"");

	return fixture;
}


Fixture *DynamicMaker::makeStaticFixture(const string& name) const
{
	if (name == "Fixture")
		return new Fixture;
	if (name == "ColumnFixture")
		return new ColumnFixture;
	if (name == "PrimitiveFixture")
		return new PrimitiveFixture;
	if (name == "ActionFixture")
		return new ActionFixture;
	if (name == "ArithmeticFixture")
		return new ArithmeticFixture;
	if (name == "Summary")
		return new Summary;
	return 0;
}



pair<string,string> DynamicMaker::splitName(const string& name)
{
	int breakPoint = name.find(".");
	if (breakPoint == string::npos) 
		return pair<string,string>(name,"");

	return pair<string,string>(
			name.substr(0,breakPoint),
			name.substr(breakPoint+1));

}

