#pragma warning(disable:4786)

#include "TransactionSource.h"
#include "Transaction.h"
#include "PaydayTransaction.h"
#include "TimeCardTransaction.h"
#include "AddEmployeeTransaction.h"
#include "NullTransaction.h"
#include "EmployeeDatabase.h"
#include "StringTokenizer.h"
#include "MockOutput.h"


TransactionSource::TransactionSource(const std::string& stringSource)
: source(stringSource), db(new EmployeeDatabase), isDone(false)
{
}


bool TransactionSource::isCommand(const std::string& commandName, const std::string& line) const
{
	return !strncmp(commandName.c_str(), line.c_str(), commandName.size());
}


Transaction *TransactionSource::getNextTransaction() 
{
	StringTokenizer tokenizer(source);
	MockOutput output;

	if (isCommand("Payday", source))
		return new PaydayTransaction(db,&output);
	if (isCommand("TimeCard", source))
		return new TimeCardTransaction(db,tokenizer.getInteger(1),tokenizer.getInteger(2));
	if (isCommand("AddEmployee", source)) {
		return new AddEmployeeTransaction(db,
			tokenizer.getString(1), 
			tokenizer.getInteger(2),
			tokenizer.getCurrencyAmount(3));
	}

	isDone = true;
	return new NullTransaction;
}

bool TransactionSource::hasMoreTransactions() const
{
	return !isDone;
}
