#pragma warning(disable:4786)

#include "StringTokenizer.h"

StringTokenizer::StringTokenizer(const std::string& source)
: source(source)
{
	std::string current;

	for(int n = 0; n < source.size(); n++) {
		if (source[n] == ',') {
			appendToken(current);
			current = "";
		}
		else
			current += source[n];
	}

	appendToken(current);
}


void StringTokenizer::appendToken(const std::string& token)
{
	std::string trimmedToken = trim(token);
	if (trimmedToken.size() > 0)
		tokens.push_back(trimmedToken);
}


std::string StringTokenizer::trim(const std::string& field)
{
	int begin = 0;
	while(begin < field.size() && isspace(field[begin]))
		begin++;

	int end = field.size() -1;
	while(end >= 0 && isspace(field[end]))
		end--;

	if (begin > field.size() || end < 0)
		return "";

	int length = end - begin + 1;
	
	return length > 0 ? field.substr(begin, length) : "";

}

int StringTokenizer::tokenCount() const
{
	return tokens.size();
}

bool StringTokenizer::inRange(int tokenNumber) const
{
	return tokenNumber >= 0 && tokenNumber < tokenCount();
}

int StringTokenizer::getInteger(int tokenNumber) const
{
	int result = 0;
	if (inRange(tokenNumber))
		result = atoi(tokens[tokenNumber].c_str());
	return result;
}

int StringTokenizer::getCurrencyAmount(int tokenNumber) const
{
	int result = 0;
	if (inRange(tokenNumber) && tokens[tokenNumber][0] == '$')
		result = atoi(&(tokens[tokenNumber].c_str())[1]);
	return result * 100;
}

std::string StringTokenizer::getString(int tokenNumber) const
{
	std::string result = "";
	if (inRange(tokenNumber))
		result = tokens[tokenNumber];
	return result;
}
